import asyncio
import sys
import datetime
import time
import click
from rich.console import Console

from src.services.lightning_service import LightningService

from ..print_value import print_value


@click.command()
@click.pass_context
def wip(ctx, **kwargs):
    """Display Boosts that have been received."""
    console: Console = ctx.obj["console"]
    console_error: Console = ctx.obj["console_error"]
    lighting_service: LightningService = ctx.obj["lightning_service"]


    try:
        asyncio.run(main(console))
    except Exception as e:
        console_error.log(e)


async def main(console):
    loop = asyncio.get_event_loop()
    queue = asyncio.Queue(100)
    ticker = Ticker()
    await asyncio.gather(
        run_ticker(ticker, queue, 3, 1),
        batcher(queue, 200, console),
    )




async def run_ticker(ticker, queue, interval, sleep_interval):
    async for _ in ticker.run(interval, sleep_interval):
        queue.put_nowait(100)


async def batcher(queue, minimum_amount, console):
    pending = 0
    while True:
        try:
            amount = await queue.get()
        except Exception as e:
            console.print(e)
        pending += amount
        if pending >= minimum_amount:
            console.print("Boosting!")
            pending = 0
        queue.task_done()


class Ticker:
    def __init__(self):
        self.event = asyncio.Event()

    async def play(self):
        self.event.set()

    async def pause(self):
        self.event.clear()

    async def run(self, interval, sleep_interval, auto_play=True):
        if auto_play:
            await self.play()
        seconds = 0
        while True:
            await self.event.wait()

            await asyncio.sleep(sleep_interval)
            seconds += sleep_interval

            await self.event.wait()

            if seconds >= interval:
                seconds = 0
                yield
